@@ -6,12 +6,14 @@
 artfimaSDF <-function(n=100, d=0, lambda=0, phi=numeric(0), theta=numeric(0), 
                      obj=NULL, plot=c("loglog", "log", "none")) {
   plot <-  match.arg(plot)
-  if ("artfima" %in% class(obj)) {
-    d <- obj$dHat
-    lambda <- obj$lambdaHat
-    phi <- obj$phiHat
-    theta <- obj$thetaHat
-  }
+  if (!is.null(obj)) {
+    if ("artfima" == class(obj)) {
+      d <- obj$dHat
+      lambda <- obj$lambdaHat
+      phi <- obj$phiHat
+      theta <- obj$thetaHat
+    }
+  }  
   lams <- 2*pi*seq(from=1/n, to=1/2, by=1/n)
   if (length(lambda)==0) {
     s <- sdfarfima(n, d=d, phi=phi, theta=theta)
@@ -87,7 +89,10 @@ sdfarfima<- function(n, d=0, phi=numeric(0), theta=numeric(0)) {
   }
   den <- as.vector(C*C+S*S)
   s1 <- num/den
-  s2 <- ifelse(length(d)==0, 1,  (2*sin(lams/2))^(-2*d))
-  #s2 <- (2*sin(lams/2))^(-2*d)
+  if (length(d)==0) {
+    s2 <- 1
+  } else {
+    s2 <- (2*sin(lams/2))^(-2*d)
+  }
   s1*s2
 }
